/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#if !defined(DEVICE__H)

#define DEVICE__H

#include <QString>
#include <QObject>
#include <qglobal.h>

class PartitionTable;
class CreatePartitionTableOperation;

class Device : public QObject
{
	Q_OBJECT
	Q_DISABLE_COPY(Device);

	friend class CreatePartitionTableOperation;

	public:
		Device(const QString& name, const QString& devicenode, qint32 heads, qint32 numSectors, qint32 cylinders, qint64 sectorSize);
		~Device();

	public:
		const QString& name() const { return m_Name; }
		const QString& deviceNode() const { return m_DeviceNode; }
		PartitionTable& partitionTable() { return *m_PartitionTable; }
		const PartitionTable& partitionTable() const { return *m_PartitionTable; }
		qint32 heads() const { return m_Heads; }
		qint32 cylinders() const { return m_Cylinders; }
		qint32 sectorsPerTrack() const { return m_SectorsPerTrack; }
		qint32 sectorSize() const { return m_SectorSize; }
		qint64 totalSectors() const { return static_cast<qint64>(heads()) * cylinders() * sectorsPerTrack(); }
		qint64 capacity() const { return totalSectors() * sectorSize(); }
		qint64 cylinderSize() const { return static_cast<qint64>(heads()) * sectorsPerTrack(); }

	protected:
		void setPartitionTable(PartitionTable* ptable) { m_PartitionTable = ptable; }
		
	private:
		QString m_Name;
		QString m_DeviceNode;
		PartitionTable* m_PartitionTable;
		qint32 m_Heads;
		qint32 m_SectorsPerTrack;
		qint32 m_Cylinders;
		qint32 m_SectorSize;
};

#endif

